/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "break.h"
#include "cb_presence.h"
#include "symbian_auto_ptr.h"
#include "contextCommon.h"
#include "bbxml.h"
#include "cbbsession.h"

const TInt CPresenceHolder::KDefaultBufferSize = 256;

EXPORT_C CPresenceHolder* CPresenceHolder::NewL(CJabberData& JabberData)
{
	CALLSTACKITEMSTATIC_N(_CL("CPresenceHolder"), _CL("NewL"));


	auto_ptr<CPresenceHolder> ret(new (ELeave) CPresenceHolder(JabberData));
	ret->ConstructL();
	return ret.release();
}

EXPORT_C CPresenceHolder::~CPresenceHolder()
{
	CALLSTACKITEM_N(_CL("CPresenceHolder"), _CL("~CPresenceHolder"));


	delete iBBSession;
	delete iListeners;
	delete iPresenceData;
}

EXPORT_C void CPresenceHolder::AddListener(MPresenceListener* Listener)
{
	CALLSTACKITEM_N(_CL("CPresenceHolder"), _CL("AddListener"));


	iListeners->AppendL(Listener);
}

EXPORT_C const CBBPresence* const CPresenceHolder::GetPresence(TInt ContactId) const
{
	CALLSTACKITEM_N(_CL("CPresenceHolder"), _CL("GetPresence"));

	return (CBBPresence*) iPresenceData->GetData(ContactId);
}

CPresenceHolder::CPresenceHolder(CJabberData& JabberData) : iJabberData(JabberData) { }

void CPresenceHolder::ConstructL()
{
	CALLSTACKITEM_N(_CL("CPresenceHolder"), _CL("ConstructL"));

	iPresenceData=CGenericIntMap::NewL();
	iPresenceData->SetDeletor(CBBPresenceDeletor);
	iListeners=CList<MPresenceListener*>::NewL();

	iSendTimeStamp = TTime();

	iBBSession=BBSession()->CreateSubSessionL(this);
	iBBSession->AddNotificationL(KIncomingPresence, ETrue);

}

/*EXPORT_C void CPresenceHolder::NewPresence(const TDesC& Nick, 
					   const TDesC& Info, const TTime & send_timestamp)
{
	CALLSTACKITEM_N(_CL("CPresenceHolder"), _CL("NewPresence"));

	TInt contact;
	contact=iJabberData.GetContactIdL(Nick);
	if (contact==KErrNotFound) return;

	// only supports new format
	TBuf<15> pv2=_L("<presencev2");
	if (Info.Left(pv2.Length()).Compare(pv2)) return;

	TPtrC8 xml8( (TUint8*)Info.Ptr(), Info.Size());

	auto_ptr<CBBPresence> data( CBBPresence::NewL() );
	auto_ptr<CSingleParser> parser( CSingleParser::NewL(data.get(), EFalse, ETrue) );
	CC_TRAPD(err, parser->ParseL(xml8) );
	if (err!=KErrNone) return;
	data->iSentTimeStamp()=send_timestamp;
	data->iSent=ETrue;

	iPresenceData->AddDataL(contact, data.get(), true);
	CBBPresence* dp=data.release();
	dp->AddRef();

	CList<MPresenceListener*>::Node* i=iListeners->iFirst;
	while (i) {
		CC_TRAPD(err, i->Item->PresenceChangedL(contact, dp));
		i->Item->Notify(_L(""));
		i=i->Next;
	}
}*/


void CPresenceHolder::NewValueL(TUint aId, const TTupleName& aName, const TDesC& aSubName, 
	const TComponentName& aComponentName, const MBBData* aData)
{
	TInt contact=iJabberData.GetContactIdL(aSubName);
	if (contact==KErrNotFound) return;

	const CBBPresence* retrieved=bb_cast<CBBPresence>(aData);

	if (!retrieved) return;
	auto_ptr<CBBPresence> data(bb_cast<CBBPresence>(retrieved->CloneL(KNullDesC)));
	if (!data.get()) User::Leave(KErrGeneral);
	data->iSent=ETrue;
	iPresenceData->AddDataL(contact, data.get(), true);
	CBBPresence* dp=data.release();
	dp->AddRef();
	CList<MPresenceListener*>::Node* i=iListeners->iFirst;

	while (i) {
		CC_TRAPD(err, i->Item->PresenceChangedL(contact, dp));
		i->Item->Notify(_L(""));
		i=i->Next;
	}	
}

void CPresenceHolder::DeletedL(const TTupleName& aName, const TDesC& aSubName)
{
	TInt contact=iJabberData.GetContactIdL(aSubName);
	if (contact==KErrNotFound) return;

	iPresenceData->DeleteL(contact);

	CList<MPresenceListener*>::Node* i=iListeners->iFirst;
	while (i) {
		CC_TRAPD(err, i->Item->PresenceChangedL(contact, 0));
		i->Item->Notify(_L(""));
		i=i->Next;
	}	
}

EXPORT_C CBBPresence* CPresenceHolder::GetPresence(TInt ContactId)
{
	CALLSTACKITEM_N(_CL("CPresenceHolder"), _CL("GetPresence"));


	return (CBBPresence*)iPresenceData->GetData(ContactId);
}

void CPresenceHolder::CBBPresenceDeletor(void* p)
{
	CALLSTACKITEMSTATIC_N(_CL("CPresenceHolder"), _CL("CBBPresenceDeletor"));

	CBBPresence* b=(CBBPresence*)p;
	b->Release();
}

