#ifndef CONTEXTMEDIACONTAINER_H
#define CONTEXTMEDIACONTAINER_H

#include <coecntrl.h>
#include <aknlists.h> 
#include "cm_storage.h"
#include <akniconarray.h>
#include <aknview.h>
#include <eikrted.h>
#include "cm_network.h"
#include <PAlbImageViewerBasic.h>
#include "medialistbox.h"
#include "videoengine.h"
#include "audioengine.h"
#include "timeout.h"
#include <aknwaitdialog.h>
#include "reporting.h"
#include "app_context.h"
#include "mplayeruicontrollerlistener.h"
#include <eikedwin.h>

IMPORT_C TInt LoadCmuiResourceL();

class MContextMediaAppUi {
public:
	virtual TUid LastViewBeforePrompt() = 0;
	virtual TVwsViewId NextView() = 0;
	virtual void SetNextView(TVwsViewId aViewId) = 0;
	virtual void SetTab(TInt tabId) = 0;
};


class OurOwnRichTextEditor;

class CContextMediaAppGeneralContainer : public CCoeControl, public MContextBase {
public: 
	CContextMediaAppGeneralContainer(CCMNetwork &aCMNetwork, CPostStorage &aStorage, TInt64 aNode, 
					CPostStorage::TSortBy aSort, CPostStorage::TOrder aOrder, CAknView * aView);
	virtual void ConstructL(const TRect& aRect) = 0;
        virtual ~CContextMediaAppGeneralContainer();

public:
	virtual TInt get_current_idx()=0;
	virtual TInt get_top_idx()=0;
	virtual void set_current_idx(TInt idx)=0;
       	virtual void set_top_idx(TInt idx)=0;
	virtual TInt get_item_count()=0;

public:
	virtual CCMPost * get_current_post()=0;
	void mark_as_read();
	virtual TBool IsCurrentPlayableMedia()=0;
	virtual TBool IsLoadingMedia()=0;
	virtual TBool CloseCurrentMedia()=0;
	
protected:
	void BaseConstructL();
	void GetVideoMbmPath(TFileName &aFilename);
	void GetAudioMbmPath(TFileName &aFilename);
	void GetUnknownMbmPath(TFileName &aFilename);

	TFileName iVideoMbmPath;
	TFileName iAudioMbmPath;
	TFileName iUnknownMbmPath;

	CPostStorage &iStorage;
	CCMNetwork &iCMNetwork;
	TInt64 iNode;
	CPostStorage::TSortBy iSort;
	CPostStorage::TOrder iOrder;
	CAknView * iView;
};

class CContextMediaAppListboxContainer : public CContextMediaAppGeneralContainer,
	public MEikListBoxObserver, public MPostNotify, public MTimeOut/*, public MBusyIndicator*/ {
public: 
	IMPORT_C CContextMediaAppListboxContainer(CCMNetwork &aCMNetwork, CPostStorage &aStorage, TInt64 aNode, 
		CPostStorage::TSortBy aSort, CPostStorage::TOrder aOrder, CAknView * aView,
		TBool aStandAlone=EFalse);
	virtual void ConstructL(const TRect &aRect);
        virtual ~CContextMediaAppListboxContainer();
private:
	void SizeChanged();
        TInt CountComponentControls() const;
        CCoeControl* ComponentControl(TInt aIndex) const;
        void Draw(const TRect& aRect) const;
	TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);

private: 
	void HandleListBoxEventL(CEikListBox* aListBox,TListBoxEvent aEventType);
	virtual void PostEvent(CCMPost* aParent, CCMPost* aChild, TEvent aEvent);

private: 
	void expired(CBase * Source);

public:
	virtual TInt get_current_idx();
	virtual TInt get_top_idx();
	virtual void set_current_idx(TInt idx);
       	virtual void set_top_idx(TInt idx);
	virtual TInt get_item_count();
	virtual CCMPost * get_current_post();
	virtual TBool CloseCurrentMedia() { return ETrue; }

public:
	virtual TBool IsCurrentPlayableMedia();
	virtual TBool IsLoadingMedia();

private:
	CContextMediaBox * iListbox;
	CAknIconArray * iIconArray;
	CContextMediaArray * iMediaTextArray;

	CTimeOut * iTimer;
	
	HBufC * iNoTitleBuf;
	HBufC * iNoItemBuf;
	HBufC * iNoVisibleItemBuf;
	HBufC * iNewThreadBuf;
	
	CEikLabel * iNoItemLabel;
	TBool	iStandAlone;
#ifdef __S60V2__
	class CAknsBasicBackgroundControlContext *iBackground;
#endif
};

class CContextMediaAppPostContainer : public CContextMediaAppGeneralContainer, public MPlayerUIControllerListener,
					public MPostNotify, public MTimeOut, public MEikEdwinObserver
{
public: 
	IMPORT_C CContextMediaAppPostContainer(CCMNetwork &aCMNetwork, CPostStorage &aStorage, TInt64 aNode, 
		CPostStorage::TSortBy aSort, CPostStorage::TOrder aOrder, CAknView * aView, 
		TBool aStandAlone=EFalse, class CTagStorage* aTagStorage=0, 
		const TDesC& aFileName=KNullDesC, MBBDataFactory * aFactory=0,
		class CAknIconArray * iTagIcons=0, class MAskForNames* aAskForNames=0);

	virtual void ConstructL(const TRect &aRect);
        virtual ~CContextMediaAppPostContainer();
private:
	void SizeChanged();
        TInt CountComponentControls() const;
        CCoeControl* ComponentControl(TInt aIndex) const;
        void Draw(const TRect& aRect) const;
	TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);
	void RestoreOriginalCBAL();
	TBool DisplayPicture();
	void DisplayPictureInnerL(TInt& aLive, class CPAlbImageViewerBasic* & localMediaDisplay);

	TBool iAutoTagsIsVisible;
	void AnimateAutoTags(TBool aIsVisible, TBool aWithBackground=EFalse);
	void ShowAutoTags(TBool aDraw);
	void DrawMediaDisplay();
	void HandleEdwinEventL(CEikEdwin* /*aEdwin*/, TEdwinEvent /*aEventType*/);
	void SetCountTextL();
public:
	virtual TInt get_current_idx();
	virtual TInt get_top_idx();
	virtual void set_current_idx(TInt idx);
       	virtual void set_top_idx(TInt idx);
	virtual CCMPost * get_current_post();
	virtual TInt get_item_count();
	virtual TBool CloseCurrentMedia();
public:
	virtual TBool IsCurrentPlayableMedia();
	virtual TBool IsLoadingMedia();
	IMPORT_C void Play();
	IMPORT_C void Pause();
	IMPORT_C void Resume();
	IMPORT_C void Stop();

private:
	void expired(CBase*source);

private: // PRIVATE MEDIA FUNCTIONS
	void PlayMediaFileL(const TDesC& aFilename);
	void PauseMediaFileL();
	void StopMediaFileL();
	void ResumeMediaFileL();

private: // video callbacks
	void InitControllerCompletedL(TInt aError);
        void PlayCompletedL(TInt aError);
	void ShowVolume(TInt volume);

private: // post callback
	void PostEvent(CCMPost* aParent, CCMPost* aChild, TEvent aEvent);

public: // Browsing related
	TBool has_next();
	TBool has_previous();   
	void display_next();
	void display_previous();
	void display_current(TInt aError=KErrNone);

private: // focus related
	void FocusPrevious();
	void FocusNext();


public:
	void save_contribution_to_post(TBool aConfirmed);

public:
	enum TMediaFileType {EUnknown, EImage, EAudio, EVideo};
	TMediaFileType iMediaFileType;

	virtual void SetPresence(CBBPresence* aPresence); // takes ownership

private: 
	TEikScrollBarModel iModel;
	CEikScrollBarFrame * iSBFrame;
	TInt iMaxScrollPos;
	TInt iCurrentScrollPos;

	CEikLabel * iLoadingLabel;
	CEikEdwin * iErrorLabel;
	OurOwnRichTextEditor * iComment;
	OurOwnRichTextEditor * iTags;
	OurOwnRichTextEditor * iSignature;
	CEikLabel * iLeftArrow;
	CEikLabel * iLeftArrowBg;
	CEikLabel * iLeftArrowBlackBg;
	CEikLabel * iRightArrow;
	CEikLabel * iRightArrowBg;
	CEikLabel * iRightArrowBlackBg;
	CEikLabel * iTimeLabel;
	CEikLabel * iTimeLabelBg;
	CEikLabel * iTimeLabelBlackBg;
	CEikLabel * iCountLabel;
	CEikLabel * iCountLabelBg;
	CEikLabel * iCountLabelBlackBg;
	CEikLabel * iVolumeIndicator;
	CEikLabel * iSelectTagLabel;
	CTimeOut * iVolumeDisplayTimeOut;
	class CNotifyWindowControl* iNotifyControl;

	HBufC * iLoadingBuf;
	HBufC * iDownloadingBuf;
	HBufC * iDefaultSignatureBuf;
	HBufC * iDefaultTagsBuf;
	HBufC * iDefaultCommentBuf;

	TBool iMediaReady;   // whether media is ready (to be displayed and/or played)
	TBool iLoadingMedia; // whether we're opening the media file for display
	TBool iPlayingMedia; // whether we currently are playing media
	TBool iMediaBrowsingOn; //in captured media mode, whether we can go left&right
	
	CPAlbImageViewerBasic * iMediaDisplay;

	CArrayFixFlat<TUint>* iPostArray;
	CCMPost	*iPost, *iParentPost;
	TInt iCurrentPostIndex;

	CVideoEngine * iVideoEngine;
	CAudioEngine * iAudioEngine;
	CTimeOut * iDisplayTimeOut; TBool iCurrentChanged;

	TFileName iFileName;

	CArrayPtrFlat< CCoeControl > * iControls;
	TBool	iStandAlone;

	MBBDataFactory * iFactory;
	class CTagStorage* iTagStorage;
	TInt	*iLive;

	class CAutoTagListBox*	iAutoTags;
	class CAutoTagArray*	iAutoTagArray;
	class CTimeOut*		iAutoTimer;
	class CBBPresence*	iPresence;
	TInt	iWaitCount;
	class CAknIconArray * iTagIcons;
	TBool	iGoneUpFromAutoTags;
	MAskForNames* iAskForNames;
};

#endif