#include "flickr_rsp.h"
#include "bberrors.h"

class CFlickrResponseImpl : public CFlickrResponse {

	TBool iOk, iDone;
	TInt iErrorCode, iPhotoId;
	HBufC* iErrorMessage;
	HBufC* iXmlErrorMessage;
	~CFlickrResponseImpl() {
		delete iErrorMessage;
		delete iXmlErrorMessage;
	}
	virtual TBool Ok() {
		return iOk;
	}
	virtual TInt ErrorCode() {
		return iErrorCode;
	}
	virtual const TDesC& ErrorMessage() {
		if (iErrorMessage) {
			return *iErrorMessage;
		} else {
			return KNullDesC;
		}
	}
	virtual const TDesC& XmlErrorMessage() {
		if (iXmlErrorMessage) {
			return *iXmlErrorMessage;
		} else {
			return KNullDesC;
		}
	}
	virtual TInt PhotoId() {
		return iPhotoId;
	}
	virtual TBool Done() {
		return iDone;
	}

	TInt iLevel;

	virtual void StartElement(const XML_Char *n,
		const XML_Char **atts) {

		iBuf.Zero();
		TPtrC name((const TUint16*) n);
		++iLevel;

		if (iLevel==1 && name.CompareF(_L("rsp"))==0) {
			const XML_Char *n, *v;
			for (;;) {
				n=*atts; ++atts;
				v=*atts; ++atts;
				if (!n || !v) break;
				TPtrC name((const TUint16*) n); TPtrC value((const TUint16*) v);
				if (name.CompareF(_L("stat"))==0) {
					if (value.CompareF(_L("ok"))==0) {
						iOk=ETrue;
					} else {
						iOk=EFalse;
					}
				}
			}
		}
		if (iLevel==2 && name.CompareF(_L("photoid"))==0) {
			iInPhotoId=ETrue;
		} else {
			iInPhotoId=EFalse;
		}
		if (iLevel==2 && name.CompareF(_L("err"))==0) {
			const XML_Char *n, *v;
			for (;;) {
				n=*atts; ++atts;
				v=*atts; ++atts;
				if (!n || !v) break;
				TPtrC name((const TUint16*) n); TPtrC value((const TUint16*) v);
				if (name.CompareF(_L("code"))==0) {
					TLex l(value);
					l.Val(iErrorCode);
				} else if (name.CompareF(_L("msg"))) {
					delete iErrorMessage; iErrorMessage=0;
					iErrorMessage=value.Alloc();
				}
			}
		}
	}

	TBool iInPhotoId;
	TBuf<30> iBuf;
	virtual void EndElement(const XML_Char *n) {
		TPtrC name((const TUint16*) n);
		--iLevel;
		if (iLevel==0) iDone=ETrue;
		if (iInPhotoId) {
			TLex l(iBuf);
			l.Val(iPhotoId);
		}
		iBuf.Zero();
	}
	virtual void CharacterData(const XML_Char *s,
		int len) {

		if (iInPhotoId) {
			if (len > iBuf.MaxLength()-iBuf.Length()) len=iBuf.MaxLength()-iBuf.Length();
			TPtrC data((const TUint16*) s, len);
			iBuf.Append(data);
		}
	}
	virtual void Error(XML_Error Code, const XML_LChar * String, long ByteIndex) {
		iDone=ETrue;
		iOk=EFalse;
		iErrorCode=KErrInvalidXml;
		delete iErrorMessage; iErrorMessage=0;
		delete iXmlErrorMessage; iXmlErrorMessage=0;
		iXmlErrorMessage=TPtrC( (const TUint16*) XML_ErrorString(Code)).Alloc();
		iErrorMessage=_L("Invalid response received from flickr").Alloc();
	}
	virtual TBool IsPermanent() {
		switch(iErrorCode) {
		case 6:
		case 96:
		case 97:
		case 98:
		case 99:
		case 100:
			return ETrue;
		}
		return EFalse;
	}

	friend CFlickrResponse;
};

CFlickrResponse* CFlickrResponse::NewL()
{
	return new (ELeave) CFlickrResponseImpl;
}
