/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#ifndef CL_PRESENCE_PUBLISHER_H_INCLUDED
#define CL_PRESENCE_PUBLISHER_H_INCLUDED 1

#include "app_context.h"
#include "ContextClientSession.h"
#include "ftp.h"
#include "xmlbuf.h"
#include "status_notif.h"
#include "notifystate.h"
#include "csd_presence.h"
#include "connectioninit.h"
#include "bbxml.h"
#include "settings.h"
#include "presencemaintainer.h"

class CBTDeviceList;

const TTupleName KFrozenPresenceTuple = { { CONTEXT_UID_CONTEXT_LOG }, 1 };
const TTupleName KUnfreezeTimeTuple = { { CONTEXT_UID_CONTEXT_LOG }, 2 };

class CPresencePublisher : public CCheckedActive, public MPresenceMaintainer,
		public MTimeOut, public MNetworkConnection, public MSettingListener {
public:
	static CPresencePublisher* NewL(MApp_context& Context, i_status_notif* CallBack, 
		MPresencePublisherListener& aListener,  CBTDeviceList *aBuddyList,
		CBTDeviceList *aLaptopBTs, CBTDeviceList *aDesktopBTs, CBTDeviceList *aPDABTs,
		class MAskForNames* aCellNames);
	~CPresencePublisher();

	void SuspendConnection();
	void ResumeConnection();
	bool ConnectionSuspended();
	bool Disabled();

	const CBBPresence* FrozenData() const;

	void FreezeL();
	void UnFreezeL();
	TBool IsFrozen();
private:
	CPresencePublisher(MApp_context& Context, i_status_notif* CallBack, 
		MPresencePublisherListener& aListener,  CBTDeviceList *aBuddyList,
		CBTDeviceList *aLaptopBTs, CBTDeviceList *aDesktopBTs, CBTDeviceList *aPDABTs,
		class MAskForNames* aCellNames);
	void ConstructL();
	void Restart();
	void SendUpdate();
	void MakePresence();
	void QueueUpdate();

	// CCheckedActive
	void CheckedRunL();
	void DoCancel();
	virtual TInt CheckedRunError(TInt aError);

	// MTimeOut
	void expired(CBase*);

	// MSettingListener
	void SettingChanged(TInt Setting);

	void GotSignificantChange(TBool frozenChanged);

	void ShowFrozen(TBool aIsFrozen);

	// data
	enum EState { EUnknown, EReconnecting, EConnecting, EConnected, EQueuingUpdate,
		ESendingUpdate, ESuspending, ESuspended, EResuming, EDisabling, EDisabled };
	EState			iCurrentState;
	enum ENextOp { ENone, ESuspend, EResume, EDisable, EEnable };
	ENextOp			iNextOp;

	bool			iNewValue;

	RContextClientSession	iSession;
	TBuf<30>		iUser, iPass, iServer;
	CTimeOut*		iWait;
	TInt			iWaitTime;
	TInt			iAP;
	CXmlBufExternalizer*	iPresence;
	HBufC*			iSendPresence;
	TTime			iSentTimeStamp;

	i_status_notif*		iCallBack;
	class CApp_context*	ctx;


	MPresencePublisherListener& iListener;
	CNotifyState		*iPresenceRunning, *iFrozenIndicator;
	class CIncomingLookupIndicator* iLookupIndicator;
};

#endif
