#include "appuibase.h"

#include "Context_logContainer.h" 

#ifndef FLICKR
#include <context_log.rsg>
#else
#include <contextflickr.rsg>
#include "cm_oldprompt.h"
#endif

#include "local_defaults.h"
#include "context_log.hrh"
#define CONTEXTNW 1
#include <avkon.hrh>
#include <aknquerydialog.h> 
#include <eikenv.h> // ieikonenv
#include <eikmenup.h>

#include <bautils.h>
#include "sms.h"
#include "cl_settings.h"
#include <eikdll.h>
#include "break.h"
#include "notifystate.h"
#include "btlist.h"
#include "transferdir2.h"
#include "viewids.h"
#include "callstack.h"
#include "reporting.h"
#include "app_context_impl.h"
#include "app_context_fileutil.h"
#include "picture_publisher.h"
#include "contextnotifyclientsession.h"
#include "csd_sms.h"
#include "cm_storage.h"
#include "mediaprompt.h"
#include "statusview.h"

#include "cu_errorui.h"
#include "emptytext.h"
#include "file_logger.h"
#include "sensorrunner.h"
#include "log_gps.h"
#include "cellnaming.h"
#include "applogview.h"
#include "raii_f32file.h"
#include "raii_apgcli.h"
#include "cellmap.h"
#include "presencemaintainer.h"
#include <eikdoc.h>
#include <hal.h>
#include <sysutil.h>
#include <aknwaitdialog.h> 

#ifndef __WINS__
const TInt KWatchDogTimeOut=5*60;
#else
const TInt KWatchDogTimeOut=1*60;
#endif

#ifndef __WINS__
_LIT(KIconFile, "c:\\system\\data\\context_log.mbm");
#else
_LIT(KIconFile, "z:\\system\\data\\context_log.mbm");
#endif

_LIT(file_prefix, "log");

_LIT(KAppEvent, "app_event");
#include "ntpconnection.h"

/*
 * The whole structure of contextappui, appuibase, CContextXAppUi
 * is a bit hacky, and brittle. Should be rewritten.
 */

TErrorHandlerResponse CContextLogAppUiBase::HandleError(TInt aError,
     const SExtendedError& aExtErr,
     TDes& aErrorText,
     TDes& aContextText)
{

#ifdef __WINS__
	if (aError==-20) {
		TInt x=0;
	}
#endif
	MContextAppUi::HandleError(aError, aExtErr, aErrorText, aContextText);
	return CAknAppUi::HandleError(aError, aExtErr, aErrorText, aContextText);
}

void CContextLogAppUiBase::LogFormatted(const TDesC& aMsg)
{
	LogAppEvent(aMsg);
}

void CContextLogAppUiBase::LogAppEvent(const TDesC& msg)
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("LogAppEvent"));

	_LIT(KApp, "app");
	TBBLongString m(msg, KApp);
	if (iLastLog) iLastLog->AddL(msg);
	if (app_log) app_log->post_new_value(&m);
}

TInt CContextLogAppUiBase::InitializationSteps()
{
	return MContextAppUi::InitializationSteps()+15;
}

void CContextLogAppUiBase::ConstructL()
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("ConstructL"));
#ifndef FLICKR
	MContextAppUi::ConstructL(AppContext(), file_prefix);
#else
	MContextAppUi::ConstructL(AppContext(), file_prefix, 3);
#endif

	state=_L("BaseConstructL");

	BaseConstructL(0x1008);
	StepDone();

	CLocalNotifyWindow::CreateAndActivateL();
	StepDone();

	state=_L("create status view");
	iStatusView = CStatusView::NewL(AppContext());
	iAppContainer = iStatusView->Container();
	AddViewL( iStatusView );      // transfer ownership to CAknViewAppUi
	SetDefaultViewL(*iStatusView);
	StepDone();

	{
		auto_ptr<CAknView> errorview(CErrorInfoView::NewL());
		AddViewL( errorview.get() );
		errorview.release();
	}

	iBBSubSession=BBSession()->CreateSubSessionL(0);

	iWatchDogTimer=CTimeOut::NewL(*this);
	iWatchDogTimer->Wait(5);

	StepDone();
	//FIXME

	state=_L("create app event");
	app_log=Clog_base_impl::NewL(AppContext(), KAppEvent, KAppEventTuple, 0);

	Reporting().SetActiveErrorReporter(this);

	StepDone();
	iBTDb=CDb::NewL(AppContext(), _L("BTDEV"), EFileRead|EFileWrite|EFileShareAny);

#ifndef NO_PRESENCE

	iBuddyBTs = CBTDeviceList::NewL(AppContext(), iBTDb->Db(), _L("BUDDIES"));
	iLaptopBTs = CBTDeviceList::NewL(AppContext(), iBTDb->Db(), _L("LAPTOPS"));
	iDesktopBTs = CBTDeviceList::NewL(AppContext(), iBTDb->Db(), _L("DESKTOPS"));
	iPDABTs = CBTDeviceList::NewL(AppContext(), iBTDb->Db(), _L("PDAS"));
#endif //NO_PRESENCE
	StepDone();

	state=_L("smsh");

	//iSmsSocket=CSmsSocket::NewL(AppContext(), _L("itse"), this);

	{
		TInt err=0;
		smsh=new (ELeave) sms;
		CC_TRAP(err, smsh->ConstructL());
		if (err!=KErrNone) {
			error(this, err, _L("Cannot create sms handler"));
			delete smsh; smsh=0;
		}
	}
	StepDone();

	LogAppEvent(_L("Starting"));
	iLastLog=CCircularLog::NewL(20, true);

	LogAppEvent(_L("Starting 1.5"));

	iLoggers=new (ELeave) CArrayPtrFlat<Mlog_base_impl>(10);
	iMediaPublishers=new (ELeave) CArrayPtrFlat<CPicturePublisher>(10);

	StepDone();
}

void CContextLogAppUiBase::ConstructAfterPresenceMaintainerL()
{
	// start the sensors after the presence maintainer, so that the presence
	// gets all the events. Should get them from the blackboard instead...
	//
#ifndef FLICKR
	iSensorRunner=CSensorRunner::NewL(AppContext(), smsh, ETrue, *this);
#else
	iSensorRunner=CSensorRunner::NewL(AppContext(), smsh, EFalse, *this);
#endif
	StepDone();
	
	iTransferDir=CTransferDir::NewL(AppContext(), *this, _L("TRANSFER"));
	StepDone();

#if !defined(REALLY_ONLY_LOGGING) && !defined(NO_PRESENCE)
	iMediaDb=CDb::NewL(AppContext(), _L("CL_MEDIA"), EFileRead|EFileWrite);
	AppContext().TakeOwnershipL(iMediaDb);
	iMediaStorage=CPostStorage::NewL(AppContext(), *iMediaDb, BBDataFactory());
	AppContext().TakeOwnershipL(iMediaStorage);
	iMediaPrompt=CMediaPrompt::NewL(this, iPresenceMaintainer, &(iStatusView->iNextViewId), AppContext(),
		*iMediaStorage, iTransferDir->Transferer(), this);
	MUploadPrompt *pr=iMediaPrompt;

#ifndef FLICKR
	TBool del = ETrue;
	Settings().GetSettingL(SETTING_OPTIONS_ENABLE, del);
	iOldPrompt = new (ELeave) TOldPrompt(del);
#else
	iOldPrompt = new (ELeave) TCMOldPrompt(*iMediaStorage, *iMediaPrompt->GetDummyNetwork(),
		TInt64(CPostStorage::ENotPublished) );
#endif

	TBool do_other_media=ETrue;
	{
		TBuf<50> url;
		Settings().GetSettingL(SETTING_PUBLISH_URLBASE, url);
		if (url.FindF(_L("flickr"))!=KErrNotFound) do_other_media=EFalse;
	}
	state=_L("create picture publisher");
	AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\nokia\\images"),
		_L("*jpg"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir));
	AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\images"),
		_L("*jpg"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir));

	if (do_other_media) {
		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\nokia\\videos"),
			_L("*3gp"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir));
		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\videos"),
			_L("*3gp"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir));

		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\nokia\\Sounds\\digital"),
			_L("*amr"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir, _L("*aac")));
		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\Sounds\\digital"),
			_L("*amr"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir, _L("*aac")));

		// n-gage
		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\Record"),
			_L("*amr"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir, _L("*aac")));
		
		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\Sounds\\digital"),
			_L("*amr"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir, _L("*aac")));	

		AddMediaPublisher(CPicturePublisher::NewL(AppContext(), *this, _L("?:\\system\\apps\\ContextNote"),
			_L("*txt"), SETTING_MEDIA_UPLOAD_ENABLE, MEDIA_PUBLISHER_CL, SETTING_PUBLISH_URLBASE, SETTING_PUBLISH_SCRIPT, *pr, *iOldPrompt, iTransferDir));
	}
#endif //REALLY_ONLY_LOGGING
	StepDone();


	LogAppEvent(_L("Starting 3"));

	state=_L("create log view");
	auto_ptr<CAppLogView> logview(CAppLogView::NewL(iLastLog));
	AddViewL(logview.get());
	iLogView=logview.release();
	StepDone();


	state=_L("create cell naming");
	iCellNaming=CCellNaming::NewL(AppContext(), smsh, this, iNetworkConnection, *this, iSensorRunner,
		*this);
	iAppContainer->SetCellNaming(iCellNaming);
	StepDone();

#ifdef FLICKR
	if (iMediaPublishers) {
		for (int i=0; i< iMediaPublishers->Count(); i++) {
			iMediaPublishers->At(i)->PublishOld();
		}
	}
#endif
	StepDone();
}

void CContextLogAppUiBase::FinalConstructL()
{
	iAppContainer->set_status(_L(""));

	iEikonEnv->SetSystem(ETrue);

	LogAppEvent(_L("watchdog"));

	// note that we've started
	expired(iWatchDogTimer);
	_LIT(KContextLogStarted, "context_log_started");
	TBBTime t(KContextLogStarted);
	t()=GetTime();
	TTime expires; expires=Time::MaxTTime();
	CC_TRAPD(serr, iBBSubSession->PutL(KStatusTuple, KContextLogStarted, &t, expires));
	state=_L("");
	
	LogAppEvent(_L("Started"));
	initialising=false;
	Reporting().SetState(KNullDesC);
	StepDone();
}

void CContextLogAppUiBase::AddMediaPublisher(CPicturePublisher* aPublisher)
{
	auto_ptr<CPicturePublisher> p(aPublisher);
	iMediaPublishers->AppendL(aPublisher);
	p.release();
	aPublisher->SubscribeL(KBaseTuple);
}

void CContextLogAppUiBase::PublishOldMedia()
{
	if (iTransferDir && iTransferDir->Transferer()) {
		iTransferDir->Transferer()->Trigger();
	}
	if (iMediaPublishers) {
		for (int i=0; i< iMediaPublishers->Count(); i++) {
			iMediaPublishers->At(i)->PublishOld();
		}
	}
}

CContextLogAppUiBase::~CContextLogAppUiBase()
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("~CContextLogAppUiBase"));
	{
	TInt err;

	{
		if (iNTPWait) iNTPWait->ProcessFinishedL();
		delete iBBSubSession;
		delete iNTPConnection;
		delete iSensorRunner;
	}

	{

		delete iCellNaming;
		delete app_log;
		Reporting().SetActiveErrorReporter(0);
	}

	{
		TTime t; t=GetTime();
		if (iLog) 
			CC_TRAP(err, iLog->new_value(CBBSensorEvent::VALUE, _L("APP"), _L("4"), t));
		if (iLoggers) {
			iLoggers->ResetAndDestroy();
		}
		delete iLoggers;
		delete iOldPrompt;
		if (iMediaPublishers) {
			iMediaPublishers->ResetAndDestroy();
		}
		delete iMediaPublishers;

		delete iMediaPrompt;

		/*
			iMediaStorage, and iMediaDb are given to appcontext and deleted by it
		*/

		delete iTransferDir;
	}
	delete smsh;
	delete iBuddyBTs;
	delete iLaptopBTs;
	delete iDesktopBTs;
	delete iPDABTs;
	delete iBTDb;

	delete iLastLog;

	delete iWatchDogTimer;

	CLocalNotifyWindow::Destroy();
#if defined(__S60V2__) && ! defined(__WINS__)
	// the cell change notification keeps the thread from
	// exiting otherwise
	{
		PrepareToExit();
		ReleaseContextAppUi();
		delete Document();
		User::Exit(iExitValue);
	}
#endif
	}
}

void CContextLogAppUiBase::HandleSystemEventL  (  const TWsEvent &    aEvent  )
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("HandleSystemEventL"));

	TApaSystemEvent event;
	event = *(TApaSystemEvent*)aEvent.EventData();

	switch ( event ) {
		case EApaSystemEventShutdown:
			LogAppEvent(_L("ext shutdown"));
			Exit();
			return;
			break;
		default:
			break;
	}
	CAknViewAppUi::HandleSystemEventL(aEvent);
}

void CContextLogAppUiBase::HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination)
{
	CallStackMgr().ResetCallStack();
	MActiveErrorReporter* rep=AppContext().GetActiveErrorReporter();
	if (rep) rep->SetInHandlableEvent(ETrue);

	CC_TRAPD(err, CAknAppUi::HandleWsEventL(aEvent, aDestination));
	if (err==KErrCancel) {
		CallStackMgr().ResetCallStack();
		return;
	}

	User::LeaveIfError(err);
}

void CContextLogAppUiBase::expired(CBase* aSource)
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("expired"));

	if (aSource==iWait) {
		MContextAppUi::expired(aSource);
	} else {
		RAFile f; f.ReplaceLA(Fs(), _L("c:\\system\\data\\context\\watch-context_log.dat"),
			EFileWrite|EFileShareAny);
		f.Write(_L8("watch"));
		iWatchDogTimer->Wait(KWatchDogTimeOut);
	}
}

TBool CContextLogAppUiBase::BaseHandleCommandL(TInt aCommand)
{
	switch ( aCommand )
	{
	case EAknSoftkeyBack:
		QueueOp(&CContextLogAppUiBase::hide);
		break;
	case EEikCmdExit:
		{
			LogAppEvent(_L("user shutdown"));
			Exit();
			break;
		}
	case Econtext_logDetailedView:
	case Econtext_logCmdStatusView:
		ActivateLocalViewL(KStatusView);
		break;

	case Econtext_logCmdAppNameCell:
		{
		TTime now; now.HomeTime();

		MBBData *d=0;
		iBBSubSession->GetL(KCellIdTuple, KNullDesC, d);
		bb_auto_ptr<MBBData> p(d);
		const CBBSensorEvent *ev=bb_cast<CBBSensorEvent>(d);
		if (ev) {
			const TBBCellId* c=bb_cast<TBBCellId>(ev->iData());
			if (!c) return ETrue;
			iCellNaming->name_cell(CAknQueryDialog::ENoTone, *c, now);
			}
		}
		break;
	case Econtext_logCmdAppNameCity:
		{
		TTime now; now.HomeTime();

		MBBData *d=0;
		iBBSubSession->GetL(KCellIdTuple, KNullDesC, d);
		bb_auto_ptr<MBBData> p(d);
		const CBBSensorEvent *ev=bb_cast<CBBSensorEvent>(d);
		if (ev) {
			const TBBCellId* c=bb_cast<TBBCellId>(ev->iData());
			if (!c) return ETrue;
			TBBCellId cell=*c;
			cell.iCellId()=0;
			cell.iMappedId()=iSensorRunner->CellMap()->GetId(cell);
			iCellNaming->name_cell(CAknQueryDialog::ENoTone, cell, now);
		}
		}
		break;
	case Econtext_logCmdBaseLog:
		iLogView->SetLog( iCellNaming->BaseLog() );
		ActivateLocalViewL(KLogViewId);
		break;
	case Econtext_logCmdAppLog:
		iLogView->SetLog( iLastLog );
		ActivateLocalViewL(KLogViewId);
		break;
	case Econtext_logCmdGPS:
		if (iSensorRunner->Gps()) iSensorRunner->Gps()->SelectDevice();
		break;
	case Econtext_logCmdGPSTest:
		if (iSensorRunner->Gps()) iSensorRunner->Gps()->TestDevice();
		break;
	case Econtext_logCmdSettingsCancel:
	case Econtext_logCmdSettingsSave:
	case Econtext_logCmdlogviewClose:
		ActivateLocalViewL(KStatusView);
		break;
	case Econtext_logCmdPublishOld:
		{
			PublishOldMedia();
		}
		break;
	case Econtext_logCmdMediaPool:
		if (iMediaPrompt) iMediaPrompt->ShowMediaPoolL();
		break;
	case Econtext_logCmdLocateViaOperator:
		{
		MBBData *d=0;
		iBBSubSession->GetL(KCellIdTuple, KNullDesC, d);
		bb_auto_ptr<MBBData> p(d);
		const CBBSensorEvent *ev=bb_cast<CBBSensorEvent>(d);
		if (ev) {
			const TBBCellId* c=bb_cast<TBBCellId>(ev->iData());
			if (!c) User::Leave(KErrNotFound);
			iCellNaming->ask_operator(*c);
		} else {
			User::Leave(KErrNotFound);
		}
		}
		break;
	case Econtext_logCmdDisableAutostart: {
		RAFile f; f.ReplaceLA(Fs(), _L("c:\\system\\data\\context\\disable_autostart.dat"),
			EFileWrite|EFileShareAny);
		f.Write(_L8("disable"));
					      }
		break;
	case Econtext_logCmdEnableAutostart: {
		User::LeaveIfError(Fs().Delete(_L("c:\\system\\data\\context\\disable_autostart.dat")));
					      }
		break;
	case Econtext_logCmdDeviceFamily: {
#ifndef __S60V2__
		TInt revision = 0;
		HAL::Get( HALData::EDeviceFamilyRev, revision );
		TBuf<20> buf=_L("Rev: 0x");
		buf.AppendNum( (TUint)revision, EHex);
		info(0, buf);
#else
		auto_ptr<HBufC8> agent8(SysUtil::UserAgentStringL());
		auto_ptr<HBufC> agent( HBufC::NewL( (*agent8).Length() ) );
		TInt state;
		TPtr p=agent->Des();
		CC()->ConvertToUnicode(p, *agent8, state);
		info(0, *agent);
#endif
					  }
		break;
	case Econtext_logCmdRereadUnread: {
		iMediaStorage->RereadUnreadL();
					  }
		break;
	case Econtext_logCmdNTP: {
		iNtpErrors=0;
		TInt ap;
		if (! Settings().GetSettingL(SETTING_IP_AP, ap) ) return ETrue;
		if (iNTPWait) return ETrue;
		delete iNTPConnection; iNTPConnection=0;
		iNTPConnection=CNTPConnection::NewL(*this);
		iNTPConnection->Sync(ap);
		iNTPWait = new (ELeave) CAknWaitDialog( (REINTERPRET_CAST(CEikDialog**,&iNTPWait))); 
		iNTPWait->SetTone( CAknNoteDialog::ENoTone ); 
		iNTPWait->SetCallback(this);
		iNTPWait->ExecuteLD(R_NTP_WAITNOTE);
		return ETrue; }

	default:
		return EFalse;
	}
	return ETrue;

}
void CContextLogAppUiBase::finished()
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("finished"));

	_LIT(got, "finished op");
	iAppContainer->set_status(got);
}

void CContextLogAppUiBase::error(CBase* source, TInt code, const TDesC& descr)
{
	iLastErrorFrom=source;
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("error"));

	if (descr.Length()>0 || code!=KErrNone) {
		TBuf<200> msg;
		msg.Append(_L("ERROR: "));
		msg.AppendNum(code);
		msg.Append(_L(" "));
		msg.Append(descr.Left(180));
		//globalNote->ShowNoteL(EAknGlobalInformationNote, descr	);
		LogAppEvent(msg);
#ifdef __WINS__
		RDebug::Print(msg);
#endif
		if (iAppContainer) iAppContainer->set_error(msg);
	} else {
		if (iAppContainer) iAppContainer->set_error(KNullDesC);
	}
}

void CContextLogAppUiBase::error(const TDesC& descr)
{
	error(this, KErrUnknown, descr);
}

void CContextLogAppUiBase::status_change(const TDesC& status)
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("status_change"));

	//globalNote->ShowNoteL(EAknGlobalInformationNote, status);
	if (status.Length()>0) {
		TBuf<200> msg;
		msg.Append(_L("STATUS: "));
		msg.Append(status.Left(190));
		LogAppEvent(msg);
	}
	if (iAppContainer) iAppContainer->set_status(status);
}

void CContextLogAppUiBase::add_cellid_name(const TBBCellId& cellid, const TDesC& name)
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("add_cellid_name"));

	iAppContainer->add_cellid_name(cellid, name);
	iPresenceMaintainer->add_cellid_name(cellid, name);

	if (iAskedFor && iAskedInto && cellid==*iAskedFor) {
		*iAskedInto=name;
		iGotAsked=ETrue;
	}
}

void CContextLogAppUiBase::start_app(TUid app, TInt view)
{
	CALLSTACKITEM_N(_CL("CContextLogAppUiBase"), _CL("start_app"));

	if (view==0) {
		RAApaLsSession ls; ls.ConnectLA();
		TThreadId tid;

		User::LeaveIfError( ls.StartDocument(_L(""), app, tid) );

	} else {
		CCoeAppUi::ActivateViewL(TVwsViewId(app, TUid::Uid(view)));
	}
}


TBool CContextLogAppUiBase::ProcessCommandParametersL(TApaCommand aCommand, 
	TFileName& aDocumentName,const TDesC8& aTail)
{
	if (aDocumentName.CompareF(_L("hide"))==0) {
		hide();
	}
	aDocumentName.Zero();
	return EFalse;
}


TBool CContextLogAppUiBase::NameCell(TInt aBaseId, const class TBBCellId& aCellId, TDes& aNameInto)
{
	iGotAsked=EFalse;
	iAskedFor=&aCellId;
	iAskedInto=&aNameInto;
	iCellNaming->name_cell(CAknQueryDialog::ENoTone, aCellId, TTime(0));
	iAskedFor=0;
	iAskedInto=0;
	return iGotAsked;
}

TBool CContextLogAppUiBase::NameCity(const class TBBCellId& aCellId, TDes& aNameInto)
{
	TBBCellId city=aCellId;
	city.iCellId()=0;
	city.iMappedId()=iSensorRunner->CellMap()->GetId(city);

	return NameCell(-1, city, aNameInto);
}

TBool CContextLogAppUiBase::GetExistingCell(TInt aBaseId, const class TBBCellId& aCellId, TDes& aNameInto)
{
	const HBufC* name=(const HBufC*)iCellNaming->get_cellid_names()->GetData(aCellId.iMappedId());
	if (!name && aBaseId>0) name=(const HBufC*)iCellNaming->get_cellid_names()->GetData(aBaseId);
	if (name) {
		aNameInto=*name;
		return ETrue;
	}
	return EFalse;
}

TBool CContextLogAppUiBase::GetExistingCity(const class TBBCellId& aCellId, TDes& aNameInto)
{
	TBBCellId city=aCellId;
	city.iCellId()=0;
	city.iMappedId()=iSensorRunner->CellMap()->GetId(aCellId);
	return GetExistingCell(-1, city, aNameInto);
}

void CContextLogAppUiBase::NTPInfo(const TDesC& aMsg)
{
	info(iNTPConnection, aMsg);
}
void CContextLogAppUiBase::NTPError(const TDesC& aMsg, TInt aErrorCode)
{
	if (iNtpErrors==0) {
		iNtpErrors++;
		TInt ap;
		Settings().GetSettingL(SETTING_IP_AP, ap);
		delete iNTPConnection; iNTPConnection=0;
		iNTPConnection=CNTPConnection::NewL(*this);
		iNTPConnection->Sync(ap);
	} else {
		iNtpErrors++;
		if (iNTPWait) iNTPWait->ProcessFinishedL();
		error(iNTPConnection, aErrorCode, aMsg);
		Reporting().ShowGlobalNote( EAknGlobalErrorNote, aMsg);
		delete iNTPConnection; iNTPConnection=0;
	}
}
void CContextLogAppUiBase::NTPSuccess(TTime aNewTime)
{
	if (iNTPWait) iNTPWait->ProcessFinishedL();

	TTime now; now.HomeTime();
	if ( aNewTime > now+TTimeIntervalMinutes(30) || aNewTime < now-TTimeIntervalMinutes(30) ) {
		TBuf<100> msg;
		msg=_L("Network time differs more than 30 minutes from current. Please check your timezone."); 
		Reporting().ShowGlobalNote( EAknGlobalErrorNote, msg);
	} else {
		_LIT(KTimeString,"%1/%2/%3 %-B%:0%J%:1%T%:2%S%.%*C4%:3%+B");
		TBuf<32> timeString1;
		aNewTime.FormatL(timeString1,KTimeString);
		TBuf<60> msg;
		msg=_L("Time set to "); msg.Append(timeString1);
		User::SetHomeTime(aNewTime);
		Reporting().ShowGlobalNote( EAknGlobalConfirmationNote, msg);
	}
	delete iNTPConnection; iNTPConnection=0;
}

void CContextLogAppUiBase::DialogDismissedL(TInt aButtonId)
{
	delete iNTPConnection; iNTPConnection=0;
}
