#include "cellnaming.h"
#include "symbian_tree.h"
#include "current_loc.h"
#include "locationing.h"
#include "alerter.h"
#include "csd_cell.h"
#include <AknQueryDialog.h> 
#include "app_context_fileutil.h"
#include <f32file.h>
#include <bautils.h>
#include "mcc.h"
#include "cl_settings.h"
#include "sensorrunner.h"
#include "cellmap.h"

#ifndef FLICKR
#include <context_log.rsg>
#else
#include <contextflickr.rsg>
#endif

#include "emptytext.h"
#include "contextappui.h"
#include "break.h"
#include "symbian_auto_ptr.h"
#include "csd_base.h"

_LIT(KCellAnnotation, "cell_ann");
_LIT(cellid_file_v1, "cellid_names.txt");
_LIT(cellid_file, "cellid_names_v2.txt");
_LIT(install_cellid_file, "cellid_names_inst_v2.txt");
_LIT(transfer_cellid_file, "cellid_names_trans_v2.txt");


class name_cell_callback : public CCheckedActive {
public:
	name_cell_callback(class CCellNamingImpl* i_ui);
	~name_cell_callback();
	void ConstructL();
	void call();
private:
	void CheckedRunL();
	void DoCancel();
	class CCellNamingImpl* ui;
	RTimer timer;
};

void delete_bufc(void* data)
{
	HBufC* p;
	p=(HBufC*)data;
	delete p;
}

class CCellNamingImpl : public CCellNaming, public MContextBase, public MNaming, public Mlogger {
private:

	CGenericIntMap* cellid_names;
	CCurrentLoc*	iCurrentLoc;
	MNaming&	iNaming;
	CAlerter*	iAlerter;
	name_cell_callback* caller;
	CArrayFixFlat<TTime>* name_times;
	CArrayFixFlat<TBBCellId>* name_names;
	TBBShortString iCity;
	TBBShortString iCountry;
	TBool iSeenLoc;
	CSensorRunner*	iSensorRunner;
	MContextAppUi&	iAppUi;

	CCellNamingImpl(MApp_context& aContext, MNaming& aNaming, MContextAppUi& aAppUi) : MContextBase(aContext),
		iNaming(aNaming), iCity(KCity), iCountry(KCountry), iAppUi(aAppUi) { }

	void ConstructL(class sms* smsh, i_status_notif* i_cb, MNetworkConnection * aPublisher,
		CSensorRunner* aSensorRunner) {
#if !defined(REALLY_ONLY_LOGGING) || defined(DO_LOCATIONING)
		if (smsh) loc=Clocationing::NewL(AppContext(), i_cb, this, smsh, aPublisher);
#endif
		iSensorRunner=aSensorRunner;
		Mlogger::ConstructL(AppContextAccess());
		SubscribeL(KLocationTuple);
		TUint cellid_file_version=2;
		{
			MoveIntoDataDirL(AppContext(), cellid_file);
			// copy the installed cellid_names over the real one, if
			// one has been installed

			TFileName install_cellid_filen, cellid_filen, cellid_filen_v1;
			install_cellid_filen.Format(_L("%S%S"), &AppDir(), &install_cellid_file);
			cellid_filen.Format(_L("%S%S"), &DataDir(), &cellid_file);
			cellid_filen_v1.Format(_L("%S%S"), &DataDir(), &cellid_file_v1);
			if (BaflUtils::FileExists(Fs(), install_cellid_filen)) {
				User::LeaveIfError(BaflUtils::CopyFile(Fs(), install_cellid_filen, cellid_filen));
				User::LeaveIfError(BaflUtils::DeleteFile(Fs(), install_cellid_filen));
			}
			TEntry e;
			if ( ( Fs().Entry(cellid_filen, e)!=KErrNone || e.iSize==0 ) 
					&& cellid_name_file_v1.Open(Fs(), cellid_filen_v1, 
					EFileShareAny | EFileStreamText | EFileRead | EFileWrite)==KErrNone) {
				cellid_file_version=1;
			}

			if (cellid_name_file.Open(Fs(), cellid_filen, 
				EFileShareAny | EFileStreamText | EFileRead | EFileWrite)!=KErrNone) {
				User::LeaveIfError(cellid_name_file.Create(Fs(), cellid_filen, 
					EFileShareAny | EFileStreamText | EFileRead | EFileWrite));
			}
		}


		cellid_names=CGenericIntMap::NewL();
		cellid_names->SetDeletor(delete_bufc);

		iCurrentLoc=CCurrentLoc::NewL(AppContext(), cellid_names);

		iAlerter=CAlerter::NewL(AppContext());
		cell_ann=Clog_base_impl::NewL(AppContext(), KCellAnnotation, KCellAnnotationTuple, 0);
		city_name=Clog_base_impl::NewL(AppContext(), KCity, KCityNameTuple, 10*60+30);
		country_name=Clog_base_impl::NewL(AppContext(), KCountry, KCountryNameTuple, 10*60+30);

		caller=new (ELeave) name_cell_callback(this);
		caller->ConstructL();

		name_times=new (ELeave) CArrayFixFlat<TTime>(2);
		name_names=new (ELeave)CArrayFixFlat<TBBCellId>(2);


		CC()->PrepareToConvertToOrFromL(KCharacterSetIdentifierIso88591, Fs());
		read_cellid_names(cellid_file_version);
		if (cellid_file_version==1) cellid_name_file_v1.Close();

		TInt pos=0;
		cellid_name_file.Seek(ESeekEnd, pos);

	}

	Mlog_base_impl * cell_ann, *city_name, *country_name;
	~CCellNamingImpl() {

		delete loc;
		delete cellid_names;
		delete iCurrentLoc;
		delete iAlerter;
		delete cell_ann;
		delete city_name;
		delete country_name;
		cellid_name_file.Close();
		delete name_times; delete name_names;
		delete caller;

	}

	void add_name_to_map(const TBBCellId& Cell, const TDesC& name) {
		HBufC* n=HBufC::NewL(name.Length());
		CleanupStack::PushL(n);
		*n=name;
		TInt id=Cell.iMappedId();
		cellid_names->AddDataL(id, (void*)n, true); 
		CleanupStack::Pop(); // n
	}
	void add_cellid_name(const TBBCellId& Cell, const TDesC& name) {
		if (Cell.iCellId()==0) {
			if (name.Length()>50) {
				iCity()=name.Left(50);
			} else {
				iCity()=name;
			}
			city_name->post_new_value(&iCity);
		}
		TBuf<50> b;
		Cell.IntoStringL(b);

		write_to_output(b);
		_LIT(tab, "\t");
		write_to_output(tab);
		write_to_output(name);
		_LIT(nl, "\n");
		write_to_output(nl);

		_LIT(KCellAnn, "cell_ann");
		TBBShortString m(name, KCellAnn);
		cell_ann->post_new_value(&m);

		add_name_to_map(Cell, name);
		TInt id=Cell.iMappedId();
		if (id==iCurrentLoc->CurrentBaseId()) {
			TTime now=GetTime();
			iCurrentLoc->now_at_location(&Cell, id, true, false, now);
		}
		iNaming.add_cellid_name(Cell, name);
	}
	bool is_named(TInt id)
	{
		CALLSTACKITEM_N(_CL("CContext_logContainer"), _CL("is_named"));

		if (cellid_names->GetData(id)) return true;
		return false;
	}


	CCircularLog*	BaseLog()
	{
		return iCurrentLoc->BaseLog();
	}

	CGenericIntMap* get_cellid_names()
	{
		return cellid_names;
	}
	CCurrentLoc* CurrentLoc()
	{
		return iCurrentLoc;
	}


	enum ui_state { IDLE, NAME_CELLS };
	ui_state current_state;

	Clocationing* loc;

	void NewSensorEventL(const TTupleName& aName, 
						const TDesC& aSubName, const CBBSensorEvent& aEvent)
	{
		if (aName==KLocationTuple) {
			const TBBLocation* loc=bb_cast<TBBLocation>(aEvent.iData());
			if (!loc) return;
			now_at_location(&(loc->iCellId), loc->iLocationId(), loc->iIsBase(), loc->iLocationChanged(),
				loc->iEnteredLocation());
		}
	}

	void ask_operator(const class TBBCellId & cell) {
		loc->GetNameL(&cell, ETrue);
	}

	TBool locationing_available() {
		return (loc && loc->LocationingAvailable());
	
	}
	void now_at_location(const TBBCellId* Cell, TInt id, 
						bool is_base, bool loc_changed, TTime time)
	{
		if (loc) loc->now_at_location(Cell, id, is_base, loc_changed, time);
		iCurrentLoc->now_at_location(Cell, id, is_base, loc_changed, time);
		if (id<0 || !Cell) return;

		GetCountryName( Cell->iMCC(), iCountry() );
		country_name->post_new_value(&iCountry);

		TBool ask_for_name=EFalse, get_name=EFalse;
		if ( loc_changed || ! iSeenLoc) {
			// don't prompt/get name from operator multiple times
			// in the same location
			Settings().GetSettingL(SETTING_PROMPT_FOR_NAMES, ask_for_name);
			get_name=ETrue;
		}
		iSeenLoc=ETrue;
		TTime now;
		now.HomeTime();
		TBool do_call=EFalse;

		TInt id_for_cell=Cell->iMappedId();

		if (is_base && ! is_named(id) && ! is_named(id_for_cell) ) {
			if (get_name && loc && loc->LocationingAvailable() && loc->LocationingAllowed()) {
				loc->GetNameL(Cell);
			} else {
				if (ask_for_name) {
					// don't insert duplicates
					TBool found=EFalse;
					for (int i=0; i < name_names->Count(); i++) {
						if ( (*name_names)[i]==*Cell ) {
							found=ETrue;
							break;
						}
					}
					if (! found ) {
						name_times->AppendL(now);
						name_names->AppendL(*Cell);
						do_call=ETrue;
					}
				}
			}
		}
		TBBCellId city=*Cell; city.iCellId()=0;
		city.iMappedId()=iSensorRunner->CellMap()->GetId(city);
		HBufC* cityname=(HBufC*)get_cellid_names()->GetData( city.iMappedId() );
		if (cityname) {
			iCity()=cityname->Des().Left(50);
		} else {
			iCity().Zero();
		}
		city_name->post_new_value(&iCity);

		if ( ask_for_name && ! cityname ) {
			TBool found=EFalse;
			for (int i=0; i < name_names->Count(); i++) {
				if ( (*name_names)[i]==city ) {
					found=ETrue;
					break;
				}
			}
			if (! found ) {
				name_times->AppendL(now);
				name_names->AppendL(city);
				do_call=ETrue;
			}
		}

		if (do_call) caller->call();
	}

	void CCellNamingImpl::name_cell(const CAknQueryDialog::TTone tone, const TBBCellId & cell, TTime time)
	{
		if ( ((CCoeAppUi*)CEikonEnv::Static()->AppUi())->IsDisplayingMenuOrDialog()) return;

		const TInt NAME_LENGTH=100;
		auto_ptr<HBufC> textData(HBufC::NewL(NAME_LENGTH));

		HBufC* existing_name=(HBufC*)cellid_names->GetData(cell.iMappedId());
		if (existing_name) textData->Des()=*existing_name;
		
		TPtr16 p=textData->Des();

		auto_ptr<HBufC> f(0);
		if (time!=TTime(0)) {
			if (cell.iCellId()!=0) {
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_CELL_CAPTION));
			} else {
				// city
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_CITY_CAPTION));
			}
		} else {
			if (cell.iCellId()!=0) {
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_OLD_CELL_CAPTION));
			} else {
				// city
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_OLD_CITY_CAPTION));
			}
		}
		auto_ptr<HBufC> pr(HBufC::NewL(f->Des().Length()+4));
		if (time!=TTime(0)) {
			TDateTime dt(time.DateTime());
			pr->Des().Format(f->Des(), dt.Hour(), dt.Minute());
		} else {
			pr->Des()=*f;
		}

		CAknTextQueryDialog* dlg = new(ELeave) CEmptyAllowingTextQuery(p, tone);
		CleanupStack::PushL(dlg);
		dlg->SetMaxLength(NAME_LENGTH);
		dlg->SetPromptL(*pr);
		CleanupStack::Pop();
		if (dlg->ExecuteLD(R_CONTEXT_LOG_NAME_INPUT) && textData->Length())
		{
			add_cellid_name(cell, *textData);
		}
	}

	void name_cells()
	{	
		CALLSTACKITEM_N(_CL("CCellNamingImpl"), _CL("name_cells"));

		RWsSession& wsSession=CEikonEnv::Static()->WsSession();
		TInt id = CEikonEnv::Static()->RootWin().Identifier();
		TApaTask task(wsSession);
		task.SetWgId(id);
		TBool hide_me = EFalse;
		if ( wsSession.GetFocusWindowGroup() != id )
		{
			task.BringToForeground();
			hide_me = ETrue;
		}
		//unhide();

		current_state=NAME_CELLS;

		// cannot play tone if we want to play a ringtone
		CAknQueryDialog::TTone tone=CAknQueryDialog::ENoTone; 
		iAlerter->AlertUser();

		{
			for (int i=0; i<name_names->Count(); i++) {
				TBBCellId& cell=(*name_names)[i];
				name_cell(tone, cell, (*name_times)[i]);
				name_times->Delete(i);
				name_names->Delete(i);
			}
			name_times->Compress();
			name_names->Compress();
		}

		if (iAlerter) iAlerter->StopAlert();

		if (hide_me) 
		{
			iAppUi.hide();
		}
		current_state=IDLE;

	}

	RFile cellid_name_file, cellid_name_file_v1;
	TBuf8<128> buf;
	
	void read_cellid_names(TUint version)
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("read_cellid_names"));

		TBuf<128> unibuf;
		TBuf<200> cell_name, cellid;
		TInt pos=0; TChar cur; TInt len;
		bool read_id=false;

		TInt state=CCnvCharacterSetConverter::KStateDefault;
		if (version==1) {
			cellid_name_file_v1.Read(buf);
		} else {
			cellid_name_file.Read(buf);
		}
		while ( (len=buf.Length()) ) {
			CC()->ConvertToUnicode(unibuf, buf, state);
			while (pos<len) {
				while (!read_id && pos<len && (cur=unibuf[pos])!=L'\t') {
					cellid.Append(cur);
					pos++;
				}
				if (pos<len && !read_id) {
					read_id=true;
					pos++; // skip tab
				}
				while (pos<len && (cur=unibuf[pos])!=L'\n') {
					cell_name.Append(cur);
					pos++;
				}
				if (pos<len) {
					pos++; //skip nl for next
					
					if (cellid.Length() && cell_name.Length()) {
						TBBCellId cell(KCell);

						if (version==1) {
							TUint id, lac;
							CCellMap::Parse(cellid, id, lac, cell.iShortName());
							cell.iCellId()=id; cell.iLocationAreaCode()=lac;
							iSensorRunner->OpMap()->NameToMccMnc(cell.iShortName(), cell.iMCC(), cell.iMNC());
							cell.iMappedId()=iSensorRunner->CellMap()->GetId(cell);
							add_name_to_map(cell, cell_name);
						} else {
							CC_TRAPD(err, cell.FromStringL(cellid));
							if (err==KErrNone) {
								cell.iMappedId()=iSensorRunner->CellMap()->GetId(cell);
								add_name_to_map(cell, cell_name);
							}
						}
					}
					cellid.Zero();
					cell_name.Zero();
					read_id=false;
				}
			}
			buf.Zero();
			pos=0;
			if (version==1) {
				cellid_name_file_v1.Read(buf);
			} else {
				cellid_name_file.Read(buf);
			}
		}
	}

	void write_to_output(const TDesC& str)
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("write_to_output"));

		TInt pos=0, len;
		len=buf.MaxLength();
		while (pos < str.Length()) {
			TInt real_len;
			if (pos+len > str.Length()) {
				real_len=str.Length()-pos;
			} else {
				real_len=len;
			}
			CC()->ConvertFromUnicode(buf, str.Mid(pos, real_len));
			pos+=len;
			cellid_name_file.Write(buf);
		}

	}


	friend name_cell_callback;
	friend CCellNaming;
	friend auto_ptr<CCellNamingImpl>;
};

CCellNaming* CCellNaming::NewL(class MApp_context& aContext, class sms* smsh,
	class i_status_notif* i_cb, class MNetworkConnection * aPublisher,
	class MNaming& aNaming, class CSensorRunner* aSensorRunner,
	class MContextAppUi& aAppUi)
{
	auto_ptr<CCellNamingImpl> ret(new (ELeave) CCellNamingImpl(aContext, aNaming, aAppUi));
	ret->ConstructL(smsh, i_cb, aPublisher, aSensorRunner);
	return ret.release();
}

name_cell_callback::name_cell_callback(CCellNamingImpl* i_ui) : 
	CCheckedActive(EPriorityIdle, _L("name_cell_callback")), ui(i_ui) { }

void name_cell_callback::ConstructL()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("ConstructL"));

	timer.CreateLocal();
	CActiveScheduler::Add(this); // add to scheduler
}

void name_cell_callback::call()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("call"));

	if (IsActive()) return;
	TTimeIntervalMicroSeconds32 w(1*1000*1000);
	timer.After(iStatus, w);
	SetActive();
}

void name_cell_callback::CheckedRunL()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("CheckedRunL"));

	if (ui) {
		ui->name_cells();
	}
}

void name_cell_callback::DoCancel()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("DoCancel"));

	timer.Cancel();
}

name_cell_callback::~name_cell_callback()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("~name_cell_callback"));

	Cancel();
	timer.Close();
}