#include "sensorrunner.h"
#include "app_context.h"

#include "log_cellid.h"
#include "log_profile.h"
#include "file_logger.h"
#include "bb_logger.h"
#include "log_sms.h"
#include "discover.h"
#include "bases.h"
#include "log_local.h"
#include "log_idle.h"
#include "log_appuse.h"
#include "log_gps.h"
#include "operatormap.h"
#include "independent.h"
#include "log_systemevent.h"
#include "log_shareddata.h"

const TInt KUidBatteryBarsValue = 0x100052D3;
const TUid KUidBatteryBars ={KUidBatteryBarsValue};
const TInt KUidChargerCStatusValue = 0x100052D7;
const TUid KUidChargerCStatus = {KUidChargerCStatusValue};

#include "keycapture.h"
#include "csd_system.h"
#include "csd_event.h"
#include "csd_battery.h"

#include "log_alarm.h"
#include "log_unread.h"
#include "log_calendar.h"
_LIT(KCharger, "charger");
_LIT(KNetworkStatus, "network_status");
_LIT(KBattery, "battery");
#include "csd_idle.h"
#include <bautils.h>
#include "callstack.h"
#include "csd_bluetooth.h"
#include "break.h"
#include <w32std.h>
#include "csd_sms.h"
#include <eikenv.h>
#include "cl_settings.h"
#include "symbian_auto_ptr.h"

#ifndef FLICKR
#include <context_log.rsg>
#else
#include <contextflickr.rsg>
#endif

class CSensorRunnerImpl : public CSensorRunner, public MContextBase {
private:
	MAppEvents& iAppEvents;
	CSensorRunnerImpl(MApp_context& aContext, MAppEvents& aAppEvents) : MContextBase(aContext),
		iAppEvents(aAppEvents) { }

	Clog_cellid* lc;
	Clog_profile* lp;
	//Clog_soundlevel* ls;
	CLog_local*	ll;
	CLog_idle*	li;
	CLog_AppUse*	lau;
	CLog_sms*	lsms;
	independent_worker keycapture, bases_worker;
	CDiscover* discoverer;
	bases* base_counter;
	CDb		*iCellDb;
	CCellMap	*iCellMap;
	COperatorMap		*iOpMap;

	TKeycaptureArgs	 iKeycaptureArgs;
	CArrayPtrFlat<Mlog_base_impl>	*iLoggers;
	CLogGps*		iGps;
	TBuf<50>	state;


	virtual CLogGps*	Gps() { return iGps; }
	virtual CCellMap*	CellMap() { return iCellMap; }
	virtual COperatorMap* OpMap() { return iOpMap; }
	virtual void Stop() {
		keycapture.stop();
		bases_worker.stop();
		if (iLoggers) {
			iLoggers->ResetAndDestroy();
		}
		delete iLoggers; iLoggers=0;
		delete base_counter; base_counter=0;
	}
	~CSensorRunnerImpl() {
		Stop();
		delete iCellMap;
		delete iCellDb;
		if (iOpMap) {
			iOpMap->Release(); 
			iOpMap=0;
		}
	}
	void ConstructL(sms* smsh, TBool aDoKeycapture) {
		iAppEvents.LogAppEvent(_L("Starting 1.2"));
		state=_L("create cell map");
		iCellDb=CDb::NewL(AppContext(), _L("CELLS"), EFileRead|EFileWrite);
		iOpMap=COperatorMap::NewL(AppContext()); 

		{
			TBool transfer_old_cell_ids=EFalse;
			if (! BaflUtils::FileExists(Fs(), _L("c:\\system\\data\\context\\cells.db"))) {
				transfer_old_cell_ids=ETrue;
			}
			iCellMap=CCellMap::NewL(AppContext(), iCellDb->Db(), iOpMap);

			if (transfer_old_cell_ids) {
				// NOTE: we must construct base_counter directly after
				// cellmap so that the mappings get transfered correctly
				// from the base db to the cell map db after upgrading

				iAppEvents.LogAppEvent(_L("Starting 2"));

				state=_L("create base counter");
				base_counter=new (ELeave) bases(AppContext());
				base_counter->ConstructL(true, iCellMap);
				// base_counter->test(iOpMap, iCellMap); // NOP on ARMI
				delete base_counter; base_counter=0;
			}
		}
		iLoggers=new (ELeave) CArrayPtrFlat<Mlog_base_impl>(10);
		{
			TInt errd;
			TRAP(errd, discoverer=CDiscover::NewL(AppContext()));
			if (errd==KErrNone) {
				iLoggers->AppendL(discoverer);
			} else {
				iAppEvents.error(this, errd, _L("Bluetooth scans not available: "));
				auto_ptr<HBufC> stack(CallStackMgr().GetFormattedCallStack(_L("AppUi")));
				iAppEvents.LogAppEvent(*stack);
			}
		}
		state=_L("create log cellid");
		lc=Clog_cellid::NewL(AppContext(), iOpMap, iCellMap);
		iLoggers->AppendL(lc);

		if (smsh) {
			state=_L("create log sms");
			lsms=CLog_sms::NewL(AppContext(), KSMS);
			iLoggers->AppendL(lsms);
			smsh->AddHandler(lsms);
		}

#ifndef FLICKR
		state=_L("create log profile");
		lp=Clog_profile::NewL(AppContext());
		iLoggers->AppendL(lp);

		{
			state=_L("create log alarm");
			auto_ptr<CLogAlarm> l(CLogAlarm::NewL(AppContext()));
			iLoggers->AppendL(l.get());
			l.release();
		}
		{
			state=_L("create log unread");
			auto_ptr<CLogUnread> l(CLogUnread::NewL(AppContext()));
			iLoggers->AppendL(l.get());
			l.release();
		}
#endif
		{
			state=_L("create log calendar");
			auto_ptr<CLogCalendar> l(0);
			HBufC* busytext=CEikonEnv::Static()->CEikonEnv::Static()->AllocReadResourceL(R_BUSY);
			CC_TRAPD(err, l.reset(CLogCalendar::NewL(AppContext(), busytext)));
			if (l.get()) {
				iLoggers->AppendL(l.get());
				l.release();
			} else {
				iAppEvents.error(this, err, _L("CLogCalendar creation failed"));
			}
		}

		state=_L("create log appuse");
		lau=CLog_AppUse::NewL(AppContext());
		iLoggers->AppendL(lau);

		if (aDoKeycapture) {
			ll=CLog_local::NewL(AppContext(), KIdle);
			iLoggers->AppendL(ll);
			iKeycaptureArgs.iKeyStatus=ll->RequestStatus();
		}

		state=_L("create system event loggers");
		Mlog_base_impl* l;
		l=CLog_SystemEvent::NewL(AppContext(), KCharger, KUidChargerCStatus, KChargerTuple); 
		iLoggers->AppendL(l);

		l=CLog_SystemEvent::NewL(AppContext(), KBattery, KUidBatteryBars, KBatteryTuple);
		iLoggers->AppendL(l);

		l=CLog_SystemEvent::NewL(AppContext(), KNetworkStatus, KUidNetworkStatus, KNetworkTuple); 
		iLoggers->AppendL(l);

#if !defined(REALLY_ONLY_LOGGING) || defined(CONTEXTNW)
		iGps=CLogGps::NewL(AppContext());
		iLoggers->AppendL(iGps); 
#endif

		if (aDoKeycapture) {
			state=_L("start keycapture");
			RWindowGroup wg=CEikonEnv::Static()->RootWin();
			iKeycaptureArgs.iParentWg=wg.Identifier();
			Settings().GetSettingL(SETTING_RIGHT_SOFTKEY_CONTEXT, iKeycaptureArgs.right_softkey_mapped);
			keycapture.start(_L("keycapture"), start_keycapture, (TAny*)&iKeycaptureArgs, 
				EPriorityAbsoluteForeground);
		}
		
#ifndef NO_BASES
		state=_L("start bases");
		bases_worker.start(_L("bases"), bases::RunBasesInThread, 0, EPriorityLess);
#endif
		iOpMap->Release(); iOpMap=0;

		state.Zero();
	}
	virtual const TDesC& State() {
		return state;
	}

	friend CSensorRunner;
	friend auto_ptr<CSensorRunnerImpl>;
};

CSensorRunner* CSensorRunner::NewL(class MApp_context& aContext, class sms* smsh,
	TBool aDoKeycapture, MAppEvents& aAppEvents)
{
	auto_ptr<CSensorRunnerImpl> r(new (ELeave) CSensorRunnerImpl(aContext, aAppEvents));
	r->ConstructL(smsh, aDoKeycapture);
	return r.release();
}

