#include "splash.h"
#include "app_context.h"
#include <symbian_auto_ptr.h>
#include <eikenv.h> 
#include <w32std.h>
#ifdef FLICKR
#include <contextflickr.mbg>
#else
#include <context_log.mbg>
#endif
#include <bautils.h>

const TInt progress_left=28;
const TInt progress_width=120;
const TInt progress_height=6;
const TInt progress_top=165;

class CSplashScreenImpl : public CSplashScreen {
private:
	TInt iSteps, iDone;
	CWsBitmap* iBitmap;
	RWindowGroup iWg;
	void ConstructL(TInt aSteps) {

		iWg=RWindowGroup(iEikonEnv->WsSession());
		User::LeaveIfError(iWg.Construct((TUint32)&iWg, EFalse));
		iWg.SetOrdinalPosition(1, ECoeWinPriorityAlwaysAtFront+1);
		iWg.EnableReceiptOfFocus(EFalse);

		CreateWindowL(&iWg);

		iSteps=aSteps;
		iBitmap=new (ELeave) CWsBitmap(iEikonEnv->WsSession());
#ifdef FLICKR
		TBuf<50> bm=_L("c:\\system\\data\\contextflickr.mbm");
#else
		TBuf<50> bm=_L("c:\\system\\data\\context_log.mbm");
#endif
#ifndef __WINS__
		if (! BaflUtils::FileExists(iEikonEnv->FsSession(), bm) ) bm.Replace(0, 1, _L("e"));
#else
		bm.Replace(0, 1, _L("z"));
#endif
#ifdef FLICKR
		User::LeaveIfError(iBitmap->Load(bm, EMbmContextflickrMeaning));
#else
		User::LeaveIfError(iBitmap->Load(bm, EMbmContext_logMeaning));
#endif

		SetExtentToWholeScreen();
	}
	~CSplashScreenImpl() {
		iWg.Close();
		delete iBitmap;
	}
	virtual void Draw(const TRect& aRect) const {
		TRgb back(127, 120, 120);
		TRgb front(216, 214, 214);
		CWindowGc& gc = SystemGc();
		gc.DrawBitmap(aRect, iBitmap);

		gc.SetPenStyle(CGraphicsContext::ENullPen);
		gc.SetBrushColor(back);
		gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
		TRect progress( TPoint(progress_left, progress_top), TSize(progress_width, progress_height) );
		gc.DrawRect(progress);

		progress.SetSize( TSize( progress_width * iDone/iSteps, progress_height ) );
		gc.SetBrushColor(front);
		gc.DrawRect(progress);
	}
	virtual void StepDone() {
		++iDone;
		CWindowGc& gc = SystemGc();
		ActivateGc();

		TRgb front(216, 214, 214);
		TRect progress( TPoint(progress_left, progress_top), TSize(progress_width, progress_height) );
		progress.SetSize( TSize( progress_width * iDone/iSteps, progress_height ) );
		gc.SetPenStyle(CGraphicsContext::ENullPen);
		gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
		gc.SetBrushColor(front);
		progress.SetSize( TSize( progress_width * iDone/iSteps, progress_height ) );
		gc.DrawRect(progress);
		iEikonEnv->Flush();
		DeactivateGc();
	}

	friend CSplashScreen;
};

CSplashScreen* CSplashScreen::NewL()
{
	return new (ELeave) CSplashScreenImpl;
}
